#include "iterdeep.h"
#include <stdio.h>
#include <stdlib.h>
#include "misc.h"
#include "alfabeta.h"
#include "genmoves.h"
#include "timectrl.h"
#include "makemove.h"
#include "parse.h"
#include "hash.h"

/* If post is set, then we will send output of the progress of the search
   to xboard. */
int post = 0;

/* The variables searches and search_depth are used for getting statistics
   about the average search depth. */
int searches = 0;
int search_depth = 0;
int total_time_aborts = 0;
extern int visited_nodes;

extern struct killers **killers;
extern int abort_search;

/* The following variables are used for controlling the time to search. */
extern int total_moves;
extern int at_move;
int nbr_researches_at_curr_level;

extern int historymovestable[2][64][64];

/* This function determines if an aspiration re-search has do be done, and if
   so, what the alpha/beta bounds should be. Altogether there are eight
   different states, where state zero means that no re-search is necessary. */
int next_state(int val, int *alpha, int *beta, int *org_alpha, int *org_beta,
	       int *state, int *re_searches) {
  if (*state == 1) {
    if (val <= *alpha) {
      *state = 2;
      printf("Failed low ");
    } else if (val >= *beta) {
      *state = 5;
      printf("Failed high ");
    } else
      *state = 0;
  } else if (*state == 2) {
    if (val <= *alpha) {
      *state = 3;
      printf("Failed low ");
    } else if (val >= *beta) {
      *state = 7;
      printf("Failed high ");
    } else
      *state = 0;
  } else if (*state == 3) {
    if (val >= *beta) {
      *state = 4;
      printf("Failed high ");
    } else
      *state = 0;
  } else if (*state == 4) {
    *state = 0;
  } else if (*state == 5) {
    if (val <= *alpha) {
      *state = 7;
      printf("Failed low ");
    } else if (val >= *beta) {
      *state = 6;
      printf("Failed high ");
    } else
      *state = 0;
  } else if (*state == 6) {
    if (val <= *alpha) {
      *state = 4;
      printf("Failed low ");
    } else
      *state = 0;
  } else if (*state == 7) {
    if (val <= *alpha) {
      printf("Failed low ");
      *state = 3;
    } else if (val >= *beta) {
      *state = 6;
      printf("Failed high ");
    } else
      *state = 0;
  }

  if (*state != 0) {
    printf("(state => %d)\n",*state);
    (*re_searches)++;
  }

  /* If there have been several re-searches the search window is made
     bigger than it would otherwise be. */
  if (*state == 0) {
    *org_alpha = *alpha = val - VALWINDOW - 15*(*re_searches);
    *org_beta = *beta = val + VALWINDOW + 15*(*re_searches);
  } else if (*state == 2) {
    *beta = *alpha + 1;
    *alpha = val - SECOND_ATTEMPT_VALWINDOW - 20*(*re_searches);
  } else if (*state == 3) {
    *alpha = -INFTY;
    //beta stays the same
  } else if (*state == 4) {
    *alpha = -INFTY;
    *beta = INFTY;
  } else if (*state == 5) {
    *alpha = *beta - 1;
    *beta = val + SECOND_ATTEMPT_VALWINDOW + 20*(*re_searches);
  } else if (*state == 6) {
    //alpha stays the same
    *beta = *beta = INFTY;
  } else if (*state == 7) {
    *alpha = minval(*alpha,*org_alpha);
    *beta = maxval(*beta,*org_beta);
  }
  return *state;
}

/* Returns the number of plies to which a complete search was done. */
int iterative_deepening_aspiration_search(struct board *board,
					  struct move_list_entry *movelist,
					  int mcount, int max_depth) {
  int alpha = -INFTY, beta = INFTY;
  int valmax = -INFTY;
  int i, j, k;
  int nbr_moves = 0;
  int depth;
  int attempt_nbr;
  char *str;
  int org_alpha, org_beta;
  int *searched_list;
  int best;
  int fail;
  int state = 1;
  int re_searches = 0;
  int legal_moves;
  struct move_list_entry *temp_movelist;
  struct move_new the_move;

  searched_list = (int *) malloc(mcount*sizeof(int));

  /* If a search or re-search takes too long we want to quit the search
     before arriving at a value within the alpha/beta bounds. This is
     done by using a temporary movelist, which is copied to the real
     movelist only after a search within the bounds. Thereby we know that
     the real movelist will always contain usable moves, so it can be
     returned to the caller at any time (i.e. when time is up) */
  temp_movelist = (struct move_list_entry *) malloc(mcount * sizeof(struct move_list_entry));
  for (i = 0; i < mcount; i++)
    temp_movelist[i] = movelist[i];

  /* We reserve space for killers up to ply == depth, and that's allright,
     because we don't use killer moves in the quiescence search. */
  killers = (struct killers **) calloc((unsigned) (max_depth+1),
				       sizeof(struct killers *));
  for (i = 0; i <= max_depth; i++)
    killers[i] = (struct killers *) calloc((unsigned) NBR_KILLERS,
					   sizeof(struct killers));

  /* Zero the history moves table. */
  for (i = 0; i < 2; i++)
    for (j = 0; j < 64; j++)
      for (k = 0; k < 64; k++)
	historymovestable[i][j][k] = 0;

  total_moves = mcount;

  /* Go through the moves in the movelist and play them by calling
     alphabeta. Go deeper and deeper iteratively. Start by searching
     to depth 2. If we only search to depth 1 it won't be discovered
     that one's king is left in check. */
  depth = 2;
  attempt_nbr = 1;
  org_alpha = alpha;
  org_beta = beta;
  abort_search = 0;
  set_time_per_move(board);
  set_nbr_moves(mcount);
  nbr_researches_at_curr_level = 0;

  the_move.old_castling_status[WHITE] = board->castling_status[WHITE];
  the_move.old_castling_status[BLACK] = board->castling_status[BLACK];
  the_move.old_passant = board->passant;
  the_move.old_moves_left_to_draw = board->moves_left_to_draw;

  while (1) {
    /* valmax has to be cleared (=-INFTY) for each new turn */
    valmax = -INFTY;
    /* Unset the searched moves before each new turn. */
    for (i = 0; i < mcount; i++) {
      searched_list[i] = 0;
    }
    fail = 0;
    legal_moves = 0;
    at_move = 0;
    for (nbr_moves = 0; nbr_moves < mcount && !fail; nbr_moves++) {
      best = get_next_root_move(board,temp_movelist,mcount,searched_list);
      if (best == -1) {
	infolog("BEST == -1");
	printf("BEST == -1\n");
	exit(1);
      }

      //makemove(board,&newpos,temp_movelist[best],depth);
      the_move.from_square = (char) get_first_bitpos(temp_movelist[best].fsquare);
      the_move.to_square = (char) get_first_bitpos(temp_movelist[best].tsquare);
      the_move.piece = (char) temp_movelist[best].piece;
      the_move.type = (char) temp_movelist[best].type;
      //move2history(board,&the_move,0);
      makemove_new(board,&the_move,depth);
      //temp_movelist[best].value = -alphabeta(board,-beta,-alpha,depth-1,1,1,0,0,0,(legal_moves == 0 ? 1 : 0));
      if (legal_moves == 0) {
	temp_movelist[best].value =
	  -alphabeta(board,-beta,-alpha,depth-1,1,1,0,0,0,1);
      } else {
	temp_movelist[best].value =
	  -alphabeta(board,-alpha-1,-alpha,depth-1,1,1,0,0,0,0);
	if (temp_movelist[best].value > alpha
	    && temp_movelist[best].value != -KINGTAKEN)
	  temp_movelist[best].value =
	    -alphabeta(board,-beta,-alpha,depth-1,1,1,0,0,0,1);
      }
      unmakemove_new(board,&the_move,depth);
      if (temp_movelist[best].value == -KINGTAKEN)
	temp_movelist[best].value = KINGTAKEN;
      if (temp_movelist[best].value != KINGTAKEN) {
	valmax = maxval(temp_movelist[best].value,valmax);
	/* Set alpha to maxval(alpha,valmax-1) rather than maxval(alpha,valmax).
	   Otherwise the following could happen: Lets say that the first
	   move gave a value of 5, and then alpha would be set to 5. Then
	   for the next move the search at a one ply deeper level gets a
	   5 for one of its moves. That would abort the search and give
	   a 5 for the root node's move number two. But it is in fact
	   possible that move number two is much worse than that. And if
	   move number one and number two get the same score, then it's
	   possible that the postanalysis function picks number two. */
	alpha = maxval(alpha,valmax-1);
	legal_moves++;
	at_move++;
      }
      searched_list[best] = 1;
      if (valmax >= beta)
	fail = 1;
    }
    fail = 0;
    set_nbr_moves(legal_moves);

    if (!abort_search && next_state(valmax,&alpha,&beta,&org_alpha,&org_beta,
				    &state,&re_searches) == 0) {
      level_completed();
      nbr_researches_at_curr_level = 0;
      printf("depth = %d\n",depth);
      if (post) {   //then send output to xboard
	printf("%d %d 0 0 ",depth,valmax);
	str = (char *) malloc(100*sizeof(char));
	for (i = 0; i < mcount; i++) {
	  if (temp_movelist[i].value == valmax) {
	    the_move.from_square = (char) get_first_bitpos(temp_movelist[i].fsquare);
	    the_move.to_square = (char) get_first_bitpos(temp_movelist[i].tsquare);
	    the_move.piece = (char) temp_movelist[i].piece;
	    the_move.type = (char) temp_movelist[i].type;
	    move2str(&the_move,str);
	    printf("%s ",str);
	  }
	}
	free(str);
	printf("\n");
      }
      /* Copy the temporary movelist into the list that will be returned. */
      for (i = 0; i < mcount; i++) {
	movelist[i] = temp_movelist[i];
      }

      if (time_is_up(0)) {
	//total_time_aborts++;
	break;
      }
      /* If there will be a draw, then we don't need to keep searching
	 any deeper. */
      if (board->moves_left_to_draw - depth <= 0 && valmax == 0)
	break;
      /* If we find a quick mate, then we don't need to bother searching
	 deeper. */
      if ((valmax <= (-INFTY + 150)) || (valmax >= (INFTY - 150)))
	break;
      /* If there is just one legal move we don't need to search deeper. */
      if (legal_moves < 2)
	break;
      if (depth >= max_depth)
	break;
      depth++;
      attempt_nbr = 1;
      state = 1;
    } else {
      nbr_researches_at_curr_level++;
    }

    if (abort_search) {
      printf("Search aborted.\n");
      depth--;   //since we didn't finish this search
      total_time_aborts++;
      break;
    }
  }
  print_elapsed_time(visited_nodes);
  clear_hashtable();
  searches++;
  search_depth += depth;

  /* These are only used in the search, and they should be cleared before
     next search. */
  board->captures[WHITE] = 0;
  board->captures[BLACK] = 0;

  abort_search = 0;
  printf("Ply = %d\n",depth);

  for (i = 0; i <= max_depth; i++) {
    free(killers[i]);
  }
  free(killers);
  free(temp_movelist);
  free(searched_list);

  return depth;
}
